#!/usr/bin/python3

# Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
# DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
#
# This code is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 only, as
# published by the Free Software Foundation.
#
# This code is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# version 2 for more details (a copy is included in the LICENSE file that
# accompanied this code).
#
# You should have received a copy of the GNU General Public License version
# 2 along with this work; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
# or visit www.oracle.com if you need additional information or have any
# questions.

import os
import sys
import os.path
import subprocess

# Use the specially built hg from build-merucrial.sh
hg = os.getenv('HG', 'mercurial/mercurial-4.1/hg')

def sh(cmd):
    return subprocess.check_output(cmd, shell=True)

class Repository:
    def __init__(self, path):
        self.path = path

    def execute(self, cmd):
        return sh('HGRCPATH="" %s -R %s --config=format.generaldelta=1 \
            --config=format.aggressivemergedeltas=1 \
            --config=ui.username=duke %s' % (hg, self.path, cmd))

    def init(self):
        sh('HGRCPATH="" %s --config=format.generaldelta=1 \
            --config=format.aggressivemergedeltas=1 init %s' % (hg, self.path))

    def pulltag(self, src_repo, tag):
        print("Pulling %s from %s" % (tag, src_repo.path))
        out = self.execute('pull -f -r %s %s' % (tag, src_repo.path))
        changes = out.decode('utf-8').split('\n')[-2]
        return changes != 'no changes found'

    def pullConvert(self, src_repo, end_rev, target_rev = None, splice_revs = None):
        if target_rev is not None and splice_revs is None:
            # By default splice in the children of the target_rev in the source repo
            # on top of the target_rev in this repo
            splice_revs = 'children(\'%s\')' % (target_rev)
            # Some of these children may be merge changesets. If so, we need to find
            # the other parent and preserve it by adding it to the splicemap.
            # Otherwise the merge changeset becomes broken.
            old_splice_parent = src_repo.nodes(target_rev)[0]
        else:
            # If called with an explicit splice_revs, assume there are no merge
            # changesets in it.
            old_splice_parent = None
        print("Converting %s to %s from %s" % (splice_revs, end_rev, src_repo.path))
        end_node = src_repo.nodes(end_rev)[0]
        splice_nodes = []
        if target_rev is not None:
            target_node = self.nodes(target_rev)[0]
            splice_nodes = src_repo.nodes(splice_revs)
        splicemap = self.path + '/.hg/splicemap-' + src_repo.shortname
        with open(splicemap, 'a') as f:
            for node in splice_nodes:
                if old_splice_parent is not None:
                    # Find the source repo parent that isn't the one we are replacing
                    # and add it to the splicemap to preserve it if this is a merge
                    # changeset.
                    p1 = src_repo.nodes('p1(%s)' % (node))[0]
                    p2 = src_repo.nodes('p2(%s)' % (node))[0]
                    if len(p2) > 0:
                        if p1 == old_splice_parent:
                            f.write("%s %s, %s\n" % (node, target_node, p2))
                            continue
                        elif p2 == old_splice_parent:
                            f.write("%s %s, %s\n" % (node, p1, target_node))
                            continue
                # Just add the one (new) parent to the splicemap
                f.write("%s %s\n" % (node, target_node))
        # Without convert.ignoreancestorcheck=1, the conversion fails for jdk7-b26 from
        # hotspot. This seems like a bug to me, but luckily this experimental option
        # has a side effect that works around it.
        out = sh('HGRCPATH="" %s --config=format.generaldelta=1 \
            --config=format.aggressivemergedeltas=1 --config=extensions.convert= convert \
            --config=convert.ignoreancestorcheck=1 --config=convert.hg.saverev=1 \
            --config=convert.skiptags=1 \
            --filemap %s --splicemap %s -r %s %s %s' \
           % (hg, src_repo.filemap, splicemap, end_node, src_repo.path, self.path))
        last_line = out.decode('utf-8').split('\n')[-2]
        if last_line == 'converting...':
            return False
        print(out.decode('utf-8'))
        return True

    def rebase(self, base, dest):
        print("Rebasing %s to %s" % (base, dest))
        self.execute('phase -f -d -r "ancestors(%s)"' % (base))
        self.execute('--config=extensions.rebase= rebase -s "ancestors(%s)" -d %s' % (base, dest))

    def pull(self, src_repo):
        print("Pulling from %s" % (src_repo.path))
        out = self.execute('pull -f %s' % (src_repo.path))
        changes = out.decode('utf-8').split('\n')[-2]
        return changes != 'no changes found'

    def merge(self):
        out = self.execute('heads -q')
        if len(out.decode('utf-8').split('\n')) > 2:
            self.execute('merge --tool internal:merge')
            self.execute('commit -m Merge')

    def tag(self, tag):
        self.execute('tag %s' % (tag))

    def update(self, tag = None):
        if tag == None:
            self.execute('update')
        else:
            self.execute('update -r %s' % (tag))

    def nodes(self, revset):
        return self.execute('log --template "{node}\n" -r "%s"' % (revset)) \
                   .decode('utf-8').strip().split('\n')

    def tags(self):
        tags = self.execute('tags').decode('utf-8').split('\n')
        tag_labels = [t.split()[0] for t in tags if t != '']
        if 'tip' in tag_labels:
            tag_labels.remove('tip')
        tag_labels.reverse()
        return tag_labels


class SrcRepository(Repository):
    def __init__(self, path, shortname, filemap, missing_tags = []):
        Repository.__init__(self, path)
        self.shortname = shortname
        self.filemap = filemap
        self.missing_tags = missing_tags


def repos_with_tag(repos, tag):
    # hotspot is missing four tags
    for repo in repos:
        if tag in repo.missing_tags:
            repos = [r for r in repos if r != repo]

    # nashorn showed up in build jdk8-b82
    if tag.startswith('jdk7'):
        repos = [r for r in repos if r.shortname != "nashorn"]
    if tag.startswith('jdk8'):
        build = tag.split('b')[1]
        if int(build) < 82:
            repos = [r for r in repos if r.shortname != "nashorn"]

    return repos

def most_recent_prev_tag(repo, tag, src_tags):
    i = src_tags.index(tag) - 1
    while src_tags[i] in repo.missing_tags:
        i = i - 1
    if tag == 'jdk8-b82' and repo.shortname == "nashorn":
        return src_tags[i], '0'
    return src_tags[i], None

def defineSrcRepos(src):
    src_repos = [
        SrcRepository(src, "top", "filemap_top.txt"),
        SrcRepository(src + "/hotspot", "hotspot", "filemap_hotspot.txt", \
                      ['jdk7-b144', 'jdk7-b145', 'jdk7-b146', 'jdk7-b147']),
        SrcRepository(src + "/jdk", "jdk", "filemap_jdk.txt"),
        SrcRepository(src + "/corba", "corba", "filemap_corba.txt"),
        SrcRepository(src + "/nashorn", "nashorn", "filemap_nashorn.txt"),
        SrcRepository(src + "/jaxp", "jaxp", "filemap_jaxp.txt"),
        SrcRepository(src + "/jaxws", "jaxws", "filemap_jaxws.txt"),
        SrcRepository(src + "/langtools", "langtools", "filemap_langtools.txt")
    ]
    return src_repos
