#!/bin/bash
#
# Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.
# DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
#
# This code is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 2 only, as
# published by the Free Software Foundation.
#
# This code is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# version 2 for more details (a copy is included in the LICENSE file that
# accompanied this code).
#
# You should have received a copy of the GNU General Public License version
# 2 along with this work; if not, write to the Free Software Foundation,
# Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
# or visit www.oracle.com if you need additional information or have any
# questions.

################################################################################
# This script creates a clone of a unified repository and applies the file moves
# to the new one repo layout. It then applies the necessary build patches to
# make it work.

mydir="$(dirname ${BASH_SOURCE[0]})"
mydir="$(cd "${mydir}" && pwd)"
USAGE="./movesrc.sh <src repo> <dest repo>"
src_repo="$1"
dest_repo="$2"

if [ -z "${src_repo}" ]; then
    echo $USAGE
    exit 1
fi
if [ ! -e "${src_repo}/.hg" ]; then
    echo "Invalid src repo: ${src_repo}"
    exit 1
fi
if [ -z "${dest_repo}" ]; then
    echo $USAGE
    exit 1
fi

BUILD_COMMIT_MESSAGE="Make it build"
MOVE_COMMIT_MESSAGE="Move files around"
TEST_COMMIT_MESSAGE="Make it test"

################################################################################
# Clone the repo
if [ ! -e "${dest_repo}" ]; then
    hg clone "${src_repo}" "${dest_repo}"
fi

cd "${dest_repo}"

################################################################################
# Move files around

if ! hg log -l3 --template "{desc}\n" | grep -q "${MOVE_COMMIT_MESSAGE}"; then

    # Make sure workspace is clean
    hg update
    hg revert -a
    hg st -nu | xargs rm -f

    # Edit .hgignore to avoid weirdness when moving files. Escape periods that
    # should be literal periods.
    # (".metadata" matches a dir in java.desktop)
    sed -i 's/^\./\\\./g' .hgignore

    mkdir -p src/hotspot \
        src/langtools \
        make/lib \
        make/gensrc \
        make/copy \
        make/gendata \
        make/launcher \
        make/rmic \
        make/hotspot \
        make/mapfiles \
        make/jdk \
        make/langtools \
        make/corba \
        make/data \
        make/nashorn \
        make/scripts \
        test/hotspot/jtreg \
        test/jdk test/langtools test/jaxp test/nashorn

    # hotspot/src
    hg mv hotspot/src/{jdk,java}.* src/
    hg mv hotspot/src/* src/hotspot/
    hg mv hotspot/.mx.jvmci src/hotspot/

    # jdk/src
    hg mv jdk/src/* src/

    # langtools/src
    hg mv langtools/src/sample src/langtools/
    hg mv langtools/src/{jdk,java}.* src/

    # corba/src
    hg mv corba/src/{jdk,java}.* src/

    # jaxp/src
    hg mv jaxp/src/* src/

    # jaxws/src
    hg mv jaxws/src/* src/

    # nashorn/src
    hg mv nashorn/src/* src/

    # hotspot/make
    hg mv hotspot/make/lib/Lib-* make/lib/
    hg mv hotspot/make/gensrc/Gensrc-* make/gensrc/
    hg mv hotspot/make/test/JtregNative.gmk make/test/JtregNativeHotspot.gmk
    hg mv hotspot/make/CompileTools.gmk make/CompileToolsHotspot.gmk
    hg mv hotspot/make/Tools.gmk make/ToolsHotspot.gmk
    hg mv hotspot/make/mapfiles/* make/mapfiles/
    hg mv hotspot/make/* make/hotspot/

    # jdk/make
    hg mv jdk/make/lib/* make/lib/
    hg mv jdk/make/copy/* make/copy/
    hg mv jdk/make/gendata/* make/gendata/
    hg mv jdk/make/gensrc/GensrcCommon.gmk make/gensrc/GensrcCommonJdk.gmk
    hg mv jdk/make/gensrc/* make/gensrc/
    hg mv jdk/make/launcher/* make/launcher/
    hg mv jdk/make/rmic/* make/rmic/
    hg mv jdk/make/test/JtregNative.gmk make/test/JtregNativeJdk.gmk
    hg mv jdk/make/CompileTools.gmk make/CompileToolsJdk.gmk
    hg mv jdk/make/Tools.gmk make/ToolsJdk.gmk
    hg mv jdk/make/mapfiles/* make/mapfiles/
    hg mv jdk/make/data/* make/data/
    hg mv jdk/make/scripts/* make/scripts/
    hg mv jdk/make/*.gmk make/
    hg mv jdk/make/* make/jdk/

    # langtools/make
    hg mv langtools/make/gendata/* make/gendata/
    hg mv langtools/make/gensrc/GensrcCommon.gmk make/gensrc/GensrcCommonLangtools.gmk
    hg mv langtools/make/gensrc/* make/gensrc/
    hg mv langtools/make/CompileInterim.gmk make/CompileInterimLangtools.gmk
    hg mv langtools/make/Tools.gmk make/ToolsLangtools.gmk
    hg mv langtools/make/data/* make/data/
    hg mv langtools/make/* make/langtools/

    # corba/make
    hg mv corba/make/copy/* make/copy/
    hg mv corba/make/gensrc/* make/gensrc/
    hg mv corba/make/* make/corba/

    # nashorn/make
    hg mv nashorn/make/BuildNashorn.gmk make/
    hg mv nashorn/make/* make/nashorn/

    # hotspot/test
    mkdir -p test/hotspot/jtreg/gtest test/hotspot/gtest
    hg mv hotspot/test/native/GTestWrapper.java test/hotspot/jtreg/gtest/
    hg mv hotspot/test/native/* test/hotspot/gtest/
    hg mv hotspot/test/* test/hotspot/jtreg/

    # jdk/test
    hg mv jdk/test/* test/jdk/

    # langtools/test
    hg mv langtools/test/* test/langtools/

    # jaxp/test
    hg mv jaxp/test/* test/jaxp/

    # nashorn/test
    hg mv nashorn/test/* test/nashorn/

    # Remove redundant license, jcheck and .hg* files
    hg rm -f */THIRD_PARTY_README */ASSEMBLY_EXCEPTION */README */.jcheck */.hgtags \
        */.hgignore */LICENSE .hgtags-top-repo

    # Remove forest utilities
    hg rm -f get_source.sh common/bin/hgforest.sh

    # Move out the contents from "common"
    mkdir -p make/autoconf make/nb_native make/src/native make/conf
    hg mv common/autoconf/* make/autoconf/
    hg mv common/nb_native/* make/nb_native/
    hg mv common/src/* make/src/native/
    hg mv common/conf/* make/conf/
    hg mv common/bin/{hide*,compare*,logger.sh,shell-tracer.sh} make/scripts/
    mkdir bin doc
    hg mv common/bin/* bin/
    hg mv common/doc/* doc/

    # Nashorn misc stuff
    mkdir -p bin/nashorn make/nashorn/buildtools make/nashorn/exclude doc/nashorn \
          src/sample/nashorn
    hg mv nashorn/bin/* bin/nashorn/
    hg mv nashorn/buildtools/* make/nashorn/buildtools/
    hg mv nashorn/exclude/* make/nashorn/exclude/
    hg mv nashorn/docs/* doc/nashorn/
    hg mv nashorn/samples/* src/sample/nashorn/

    # Remove old build files from jaxp and jaxws
    hg rm jaxws/* jaxp/*

    hg commit -m "${MOVE_COMMIT_MESSAGE}"
fi

################################################################################
# Patch build to work

if ! hg log -l3 --template "{desc}\n" | grep -q "${BUILD_COMMIT_MESSAGE}"; then

    # Make sure workspace is clean
    hg update
    hg revert -a
    hg st -nu | xargs rm -f

    #cp -r "${mydir}/build-files/"* .
    #hg add .

    for patch in $(find "${mydir}/build-patches" -name "*.patch"); do
        patch -p1 < "${patch}"
        if [ $? != 0 ]; then
            echo "Failed to apply ${patch}"
            exit 1
        fi
    done

    find make/autoconf -type f | xargs sed -i \
        -e 's!{JDK_TOPDIR}/src!{TOPDIR}/src!g' \
        -e 's!{HOTSPOT_TOPDIR}/test!{TOPDIR}/test/hotspot!g' \
        -e 's!HOTSPOT_TOPDIR/test/native!{TOPDIR}/test/hotspot/gtest!g' \
        -e 's!HOTSPOT_TOPDIR/test!{TOPDIR}/test/hotspot!g' \
        -e 's!HOTSPOT_TOPDIR/src/jdk!{TOPDIR}/src/jdk!g' \
        -e 's!hotspot/src/jdk!src/jdk!g' \
        -e 's!{SRC_ROOT}/jdk/src!{TOPDIR}/src!g' \
        -e 's!/common/autoconf!/make/autoconf!g' \
        -e 's!common/bin/compare.sh!make/scripts/compare.sh!' \
        -e 's!common/bin/logger.sh!make/scripts/logger.sh!' \
        -e 's!common/src!make/src/native!' \
        #

    sed -i -e 's!/common/autoconf!/make/autoconf!' configure
    sed -i -e 's!/../common/autoconf!/autoconf!' make/conf/jib-profiles.js
    sed -i -e 's!common/doc!doc!' README

    find make */make -type f | xargs sed -i \
        -e 's!IncludeCustomExtension, jdk,!IncludeCustomExtension, ,!g' \
        -e 's!\(IncludeCustomExtension,\) hotspot, \(gensrc/Gensrc-jdk\)!\1 , \2!g' \
        -e 's!IncludeCustomExtension, hotspot, !IncludeCustomExtension, , hotspot/!g' \
        -e 's!(JDK_TOPDIR)/test!(TOPDIR)/test/jdk!g' \
        -e 's!(JDK_TOPDIR)/make/src!(TOPDIR)/make/jdk/src!g' \
        -e 's!(JDK_TOPDIR)!(TOPDIR)!g' \
        -e 's!(CORBA_TOPDIR)/src!(TOPDIR)/src!g' \
        -e 's!(CORBA_TOPDIR)/make/src!(TOPDIR)/make/corba/src!g' \
        -e 's!(CORBA_TOPDIR)/make!(TOPDIR)/make!g' \
        -e 's!(LANGTOOLS_TOPDIR)/src!(TOPDIR)/src!g' \
        -e 's!(LANGTOOLS_TOPDIR)/make/src!(TOPDIR)/make/langtools/src!g' \
        -e 's!(LANGTOOLS_TOPDIR)/make/tools!(TOPDIR)/make/langtools/tools!g' \
        -e 's!(LANGTOOLS_TOPDIR)/make!(TOPDIR)/make!g' \
        -e 's!(NASHORN_TOPDIR)/src!(TOPDIR)/src!g' \
        -e 's!(NASHORN_TOPDIR)/make!(TOPDIR)/make!g' \
        -e 's!(NASHORN_TOPDIR)/buildtools!(TOPDIR)/make/nashorn/buildtools!g' \
        -e 's!(HOTSPOT_TOPDIR)/src/jdk!(TOPDIR)/src/jdk!g' \
        -e 's!(HOTSPOT_TOPDIR)/src/\$!(TOPDIR)/src/\$!g' \
        -e 's!(HOTSPOT_TOPDIR)/src!(TOPDIR)/src/hotspot!g' \
        -e 's!(HOTSPOT_TOPDIR)/test/native_!(TOPDIR)/test/hotspot/jtreg/native_!g' \
        -e 's!(HOTSPOT_TOPDIR)/test/native!(TOPDIR)/test/hotspot/gtest!g' \
        -e 's!(HOTSPOT_TOPDIR)/test!(TOPDIR)/test/hotspot/jtreg!g' \
        -e 's!(HOTSPOT_TOPDIR)/make/mapfiles!(TOPDIR)/make/mapfiles!g' \
        -e 's!(HOTSPOT_TOPDIR)/make!(TOPDIR)/make/hotspot!g' \
        -e 's!GensrcCommon.gmk!GensrcCommonJdk.gmk!g' \
        -e 's!(SRC_ROOT)!(TOPDIR)!g' \
        -e 's!include \(.*/\)\{0,1\}Tools.gmk!include \1ToolsJdk.gmk!g' \
        -e 's!-gensrc-jdk!-gensrc-src!g' \
        -e 's!-gensrc-hotspot!-gensrc-src!g' \
        -e 's!common/bin/shell!make/scripts/shell!' \
        -e 's!common/bin/jib.sh!bin/jib.sh!' \
        -e 's!common/bin/compare!make/scripts/compare!' \
        #

    bash make/autoconf/autogen.sh
    hg commit -m "${BUILD_COMMIT_MESSAGE}"
    hg st -nu | xargs rm -f
fi

################################################################################
# Patch test to work

if ! hg log -l3 --template "{desc}\n" | grep -q "${TEST_COMMIT_MESSAGE}"; then

    # Make sure workspace is clean
    hg update
    hg revert -a
    hg st -nu | xargs rm -f

    #cp -r "${mydir}/test-files/"* .
    #hg add .

    for patch in $(find "${mydir}/test-patches" -name "*.patch"); do
        patch -p1 < "${patch}"
        if [ $? != 0 ]; then
            echo "Failed to apply ${patch}"
            exit 1
        fi
    done

    ls test/*/Makefile | xargs sed -i \
        -e 's!../../test/TestCommon.gmk!../TestCommon.gmk!' \
        #
    ls test/*/*/Makefile | xargs sed -i \
        -e 's!../../test/TestCommon.gmk!../../TestCommon.gmk!' \
        #

    hg commit -m "${TEST_COMMIT_MESSAGE}"
    hg st -nu | xargs rm -f
fi

################################################################################
